# SPDX-FileCopyrightText: 2024 Helmholtz-Zentrum Dresden-Rossendorf (HZDR)
# SPDX-License-Identifier: MIT

from matplotlib import pyplot
from matplotlib.gridspec import GridSpec
from pandas import Timedelta, Series

def plot_column(column: Series, window_size: Timedelta = Timedelta(days=7)):
    """ Prepare a default plot for the data in a column.

        Prepare a plot using matplotlib.pyplot.
        You can use pyplot to either show or save the generated figure
        afterwards or modify it further.
        It will plot the data itself, overlaid with a smoothed curve obtained
        by using a rolling mean.
        Dashed lines will indicate the minimum, mean and maximum values.

        Args:
            column:
                A pandas Series, containing the measurements.
                The index should be timestamps.
            window_size:
                A Timedelta indicating the width of the rolling window used to
                plot the smoothed version of the original data.
                (Default: 7 days)
    """

    # Generate the data based on the rolling mean
    rolling_mean = column.rolling(window_size, center=True).mean()

    pyplot.ylabel(column.name)
    pyplot.plot(column, color="lightblue", alpha=0.5)
    pyplot.plot(rolling_mean, label="7-Day Mean")
    pyplot.grid(axis="x")

    for y_value in [column.min(), column.mean(), column.max()]:
        pyplot.axhline(
            y=y_value, color="orange",
            linestyle="dashed", linewidth=0.75
        )

    # Idea: add right side axis with notable values as ticks

def plot_column_statistics(column: Series):
    """ Prepare a box plot to show statistical properties of a data column.

        Prepare a plot using matplotlib.pyplot.
        You can use pyplot to either show or save the generated figure
        afterwards or modify it further.
        Since box plots can not deal with missing values,
        these will be omitted.

        Args:
            column:
                A pandas Series, containing the data to do the
                statistical evaluation on.
    """

    # Pyplot boxplots can not deal with missing values
    # so we need to filter them out

    data_present = column.notna()

    pyplot.boxplot(
        column[data_present],
        flierprops=dict(marker=".")
    )

def plot_evaluation(dataframe, column_labels):
    """ Plot an evaluation of a given set of columns from a data set.

        Each dataset column will become a row within the figure.
        On the left side of each row, the actual data will be plotted
        and on the right, a boxplot will show the significant statistical
        properties.

        Args:
            dataframe:
                The pandas dataframe which contains the rows to be plotted
            column_labels:
                A sequence of labels, representing the data columns
                to be included in the figure.
    """

    plot_rows = len(column_labels)

    grid_layout = GridSpec(
        nrows=plot_rows, ncols=2,
        wspace=0, hspace=0.1,
        width_ratios=[5,1]
    )
    figure = pyplot.figure(figsize=(12, 3 * plot_rows))
    for plot_row_index, label in enumerate(column_labels):
        column = dataframe[label]

        # Lineplot on the left
        axes = figure.add_subplot(grid_layout[2 * plot_row_index])
        plot_column(column)

        # Top plot should have its x-axis on the top,
        # plots in the middle should not have an x-axis at all
        if plot_row_index == 0:
            axes.get_xaxis().set_label_position("top")
            axes.get_xaxis().tick_top()
        elif plot_row_index != plot_rows - 1:
            axes.get_xaxis().set_ticklabels([])

        # Boxplot on the right
        axes = figure.add_subplot(grid_layout[2 * plot_row_index + 1])
        plot_column_statistics(column)

        # No need for an x-axis here
        # Move the y-axis to the right
        axes.get_xaxis().set_visible(False)
        axes.get_yaxis().set_label_position("right")
        axes.get_yaxis().tick_right()
